;**********************************************************************
;   Nabjeka s termistorem                                           *
;**********************************************************************
;    Filename:	    termo.asm                                         *
;    Author:        Vladimr Hejtmnek                                                  *
;**********************************************************************
; Nabjeka s hldnm teploty. M teplotu, pot as. Vypne nabjen
; pi zven teploty o delta T nebo po nastavenm ase. Na displeji stdav
; teplota nabjec as v procentech.
; 
;                 ----------------------  
;           +5V --|Vdd              Vss|-- GND, NTC
;               --|GP5/oscin    GP0/PGD|-- strobe
;      R1-, NTC --|GP4/oscout   GP1/PGC|-- data, vstup
;           Vpp --|GP3/MCLR/Vpp GP2/INT|-- takt, R1+
;                 ----------------------
;**********************************************************************

	list      p=12f675           ; list directive to define processor
	#include <p12f675.inc>        ; processor specific variable definitions

	errorlevel  -302              ; suppress message 302 from list file

	__CONFIG   _CP_OFF & _CPD_OFF & _BODEN_OFF & _MCLRE_OFF & _WDT_ON & _PWRTE_ON & _INTRC_OSC_NOCLKOUT 




;***** VARIABLE DEFINITIONS
w_temp        EQU     0x20        ; variable used for context saving 
status_temp   EQU     0x21        ; variable used for context saving
cdes	equ	22h		;slo destky
cjed	equ	23h		;slo jednotky
tmp		equ	24h
disw	equ	25h
sdbuf	equ	26h
flags	equ	27h
timer	equ	28h
tmpr	equ	29h		;teplota v 1 byte
delic	equ	2ah
ndes	equ	31h		;displej destky
njed	equ	32h		;displej jednotky

	#define	DDAT	GPIO,1		; displej data
	#define	DCLK	GPIO,2		; displej takt
	#define	DSTR	GPIO,0		; displej pepis
	#define	OUTP	GPIO,1		; vstup zen nabjen
	#define	NTCP	GPIO,2		; napjen termistor
	#define	JUMP	GPIO,5		; pin s jumperem
	#define	F05S	flags,0		; zpracovan peruen od TMR1
	#define	FOUT	flags,1		; nabjen 1-ano, 0-ne, hotovo
	#define	FLED	flags,7		; stav LED (test)

;**********************************************************************
		ORG     0x000             ; processor reset vector
	goto    init              ; go to beginning of program


		ORG     0x004             ; interrupt vector location
	movwf   w_temp            ; save off current W register contents
	movf	STATUS,w          ; move status register into W register
	movwf	status_temp       ; save off contents of STATUS register

	bcf     STATUS,RP0		;Bank 0
	btfss	PIR1,TMR1IF		;je to peruen od asovae?
	goto	out				;ne - tak pry
	bcf		PIR1,TMR1IF		;vynuluj pznak peruen


;korekce asovae aby peruen bylo za 500 000 Tcy(s) = 8 x 62500 (nastaven 0b-de)
;zmna na 450 000 Tcy = 8 x 56250 (nastaven 24-48)
	bcf		T1CON,TMR1ON	;vypni asova
	decf	timer,f			;-1
	bsf		F05S			;pznak zpracovanho peruen
	nop						;asov korekce
	goto	$+1				;asov korekce
	goto	$+1

	movlw	048h
;	movlw	0deh			;65536 - 62500 + 16/8 = 3036 + 2 = 0bdeh
							;8/8 -> 8 instrukc (Tcy), peddlika 8 -> ztrat se jeden impuls
	addwf	TMR1L,F
	btfsc	STATUS,C		;peteklo?
	incf	TMR1H,F
	movlw   24h
;	movlw   0bh
    addwf   TMR1H,F
	bsf		T1CON,TMR1ON	;zapni asova

out		;nvrat s obnovou registr
	movf    status_temp,w     ; retrieve copy of STATUS register
	movwf	STATUS            ; restore pre-isr STATUS register contents
	swapf   w_temp,f
	swapf   w_temp,w          ; restore pre-isr W register contents
	retfie                    ; return from interrupt


; these first 4 instructions are not required if the internal oscillator is not used
init
	call    0x3FF			; retrieve factory calibration value
	bsf     STATUS,RP0		; set file register bank to 1 
	movwf   OSCCAL			; update register with factory cal value
	movlw	07fh			;zapni WPU
	movwf	OPTION_REG
	movlw	020h				;pull-up jen na GP5
	movwf	WPU
	bcf     STATUS,RP0		; set file register bank to 0

; kompartor off
	movlw	007h			;vypne kompartor zpisem do CMCON
	movwf	CMCON			;vvody GP0 a GP1 pak lze pout jako I/O

; A/D set
	movlw	0dh				;left justified, Vref=Vdd, AN3, stop, AD on
	movwf	ADCON0
	bsf		STATUS,RP0		;Bank 1
	movlw	18h				;fosc/8, ANS3
	movwf	ANSEL			;vypne vstupy pro AD, vvody GP0, GP1 a GP2
							;pak lze pout jako I/O. GP4 analog. vstup
; timer T1 set
	bsf		PIE1,TMR1IE		;povol peruen od TMR1 (bank 1)
	bsf		INTCON,PEIE		;povol peruen od perif. (bank 0 i 1)
	bsf		INTCON,GIE		;globln peruen povoleno
	bcf     STATUS,RP0		;Bank 0
	movlw	0ffh
	movwf	TMR1H			;nastav asova kousek ped peteen
	clrf	TMR1L
	movlw	31h				;peddlika 8, intern takt, timer zapnut
	movwf	T1CON

; port and variable set
	bsf		STATUS,RP0		;Bank 1
	movlw	038h			;Set GP0,1,2 jako vstup, GP3,4,5 vstupy
	movwf	TRISIO			;
	bcf     STATUS,RP0		;Bank 0
	call	3f0h		;dlka nabjen
	movwf	timer		;nov cyklus
	btfss	STATUS,4	;byl reset od WDT?
	goto	main		;ano, nic nenastavuj
	bsf 	FOUT		;ne - nastav vstup do 1
	clrf	ndes		;nuluj destky
	clrf	njed		;nuluj jednotky
	clrf	disw		;nuluj pepna zobrazen
	movlw	.8
	movwf	delic		;delika 8 na n.3,6 sekundy

; inicializace osciltoru nen poteba

main
	clrwdt				;nastav TO ve STATUS do "1" - oeten po restartu WDT
	btfss	F05S		;bylo peruen?
	goto	main		;ne
	bcf		F05S		;nuluj pznak

	btfss	JUMP		;jen teplota, kdy jumper (pin je L)
	goto	warm

		;ta asu
	movf	timer,f		;obnov pznak Z
	btfss	STATUS,Z
	goto	warm		;nen nula
	call	3f0h		;dlka nabjen
	movwf	timer		;nov cyklus
	decfsz	delic,f
	goto	warm		;deli nen nula, nic se nepipotv
	bsf 	delic,3		;nastav dli zase na 8
	btfss	FOUT		;nabj se?
	goto	warm		;ne, u je hotovo

	call	count100
	btfsc	STATUS,Z	;u je 100?
	bcf 	FOUT		;ano - konec nabjen

		;tady pokraujeme menm a testem teploty
warm
	bsf 	NTCP		;napt pro termistor
	call	delay2		;zpodn pro ustlen napt
	bsf 	ADCON0,GO	;start AD pevodu - trv 26 Tcy
	btfsc	ADCON0,GO	;hotovo?
	goto	$-1
	bcf 	NTCP		;vypni napjen termistoru
	movlw	02h
	movwf	PCLATH		;horn st adresy
	movfw	ADRESH		;test na 0 - zkrat
	btfsc	STATUS,Z	;Pi nule se teplota netestuje?
	goto	$+5			;pesko	
	call	3f1h		;max teplota
	subwf	ADRESH,w		;
	btfss	STATUS,C	;teplota pekroena?
	bcf 	FOUT		;ano - konec nabjen
	movfw	ADRESH		;
	call	table		;pevede vsledek na teplotu
	clrf	PCLATH
	movwf	tmpr		;ulo teplotu

	btfss	JUMP		;jen teplota, kdy jumper (pin je L)
	goto	thermometer

		;co budeme ukazovat
	movlw	03h			;pepna zobrazen: 000 - znak "c ", 001,010,011 - postup v %
	andwf	disw,w		;pepna zobrazen: 100 - znak "t ", 101,110,111 - teplota v C
	btfsc	STATUS,Z	;doln 2 bity jsou 0
	goto	info		;zobraz "c " nebo "t " nebo "F "
	btfsc	disw,2		;testuj bit 2 - pepn postup (0)/teplota (1)
	goto	dtmpr		;na zobrazen teploty
	movfw	njed		;kopruj jednotky procent pro zobrazen
	movwf	cjed
	movfw	ndes		;kopruj destky procent pro zobrazen
	movwf	cdes
output
	call	dispn		;zobraz slo
	btfsc	FOUT		;rozsvcen LED na vstupu
	bsf 	OUTP
	incf	disw,f		;pepna zobrazenho daje
	goto	main

info
	clrw
	call	write4094	;zhasni jednotky
	movlw	08ch		; znak "c"
	btfss	FOUT		;testuj je-li hotovo -> ano zobraz "F "
	movlw	0e8h		; znak "F"
	btfsc	disw,2		;testuj bit 2 - pepn postup (0)/teplota (1)
	movlw	0cch		; znak "t"
	call	write4094	;zapi "c " nebo "t " na destky
	bsf		DSTR		;zobraz
	bcf		DSTR
	goto	output+1

dtmpr
	movfw	tmpr		;pevod 1 bajtu na dva plbajty
	movwf	cjed		;doln
	swapf	tmpr,w
	movwf	cdes		;horn
	goto	output		;a zobraz

thermometer
	movfw	tmpr		;pevod 1 bajtu na dva plbajty
	movwf	cjed		;doln
	swapf	tmpr,w
	movwf	cdes		;horn
	call	dispn		;zobraz slo
	incf	timer,f		;+1 kompenzace potn asu
	goto	main

		

;----------------podprogramy-----------------

count100
	incf	njed,f			;+1
	movlw	0ah				;10 do W
	xorwf	njed,w			;je v jednotkch 10?
	btfss	STATUS,Z		;kdy je 10 je Z=1
	retlw	00h 			;nen 10, tak nvrat slo +1
	clrf	njed			;vynuluj jednotky
	incf	ndes,f			;destky +1
	movlw	0ah				;10 do W
	xorwf	ndes,w			;je v destkch 10?
	btfsc	STATUS,Z		;kdy je 10 je Z=1
	retlw	0fh 			;je 100, nvrat s pznakem W=F a Z=1
	retlw	00h 			;nen 100, tak nvrat slo +1

delay2		;zpodn 20 Tcy i s call a return
	movlw	.5			;5
	movwf	tmp
	decfsz	tmp,f		;
	goto	$-1			;
	return

dispn
	movfw	cjed		;jednotky
	andlw	0fh			;jen spodn 4 bity
	call	znak		;kter segmenty?
	call	write4094	;zapi

	movfw	cdes		;destky
	andlw	0fh			;jen spodn 4 bity
	call	znak		;kter segmenty?
	call	write4094	;zapi

	bsf		DSTR		;zobraz
	bcf		DSTR
	return
	
write4094	; data na zapsn do 4094 ve W
			; Q8 nejni, Q1 nejvy bit
	movwf	sdbuf
	bcf		DCLK
	movlw	.8
	movwf	tmp
write_loop
	rrf		sdbuf,f		;pesu nejni bit do carry
	bcf		DDAT
	btfss	STATUS,C	;test C
	bsf		DDAT		;kdy C=0, zapi 1, segment nesvt
	bsf		DCLK		;zapisovac impuls (toggle DCLK)
	bcf		DDAT		;nulovn vstupu kvli sdlen vvodu
	bcf		DCLK
	decfsz	tmp,f		;smyka 8x (loop 8 times)
	goto	write_loop
	rrf		sdbuf,f		; vrt pvodn obsah - mono vypustit
	return

znak	;poad bit g, f, a, b, e, d, c, dp - (posuv. reg. s 4094)
		;poslat od nejniho bitu, segment svt na log.1
	addwf	PCL,f
	retlw	07eh	; znak 0
	retlw	012h	; znak 1
	retlw	0bch	; znak 2
	retlw	0b6h	; znak 3
	retlw	0d2h	; znak 4
	retlw	0e6h	; znak 5
	retlw	0eeh	; znak 6
	retlw	032h	; znak 7
	retlw	0feh	; znak 8
	retlw	0f6h	; znak 9
	retlw	080h	; znak "-"
	retlw	0dah	; znak "H"
	retlw	008h	; znak "i"
	retlw	04ch	; znak "L"
	retlw	08eh	; znak "o"
	retlw	000h	; znak " "

;	retlw	0fah	; znak "A"
;	retlw	0ceh	; znak "b"
;	retlw	08ch	; znak "c"
;	retlw	09eh	; znak "d"
;	retlw	0ech	; znak "E"
;	retlw	0e8h	; znak "F"
;	retlw	0b4h	; znak "stupe"
;	retlw	0cch	; znak "t"

		org 1ffh
table		;pevod bin sla z AD pevodnku na teplotu
			;dt = retlw (v programov pamti)
	addwf	PCL,f
;pevodn tabulka pro termistor 10k B=3380K, pull-up 3k9
;
;	dt	0aah	; "--"
;	dt	0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc	;01-0f "Hi"
;	dt	0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc	;10-1f "Hi"
;	dt	0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc	;20-2f "Hi"
;	dt	0bc,0bc,0bc,0bc,99,98,97,97,96,95,94,93,92,92,91,90	;30-3f
;	dt	89,88,88,87,86,85,85,84,83,83,82,81,81,80,79,79	;40-4f
;80
;	dt	78,77,77,76,76,75,74,74,73,73,72,71,71,70,70,69	;50-5f
;	dt	69,68,67,67,66,66,65,65,64,64,63,63,62,62,61,61	;60-6f
;	dt	60,60,59,59,58,58,57,57,56,56,55,55,54,54,53,53	;70-7f
;	dt	52,52,51,51,50,50,49,49,48,48,48,47,47,46,46,45	;80-8f
;	dt	45,44,44,43,43,42,42,41,41,40,40,40,39,39,38,38	;90-9f
;160
;	dt	37,37,36,36,35,35,34,34,33,33,32,32,31,31,30,30	;a0-af
;	dt	29,29,28,28,27,27,26,26,25,25,24,24,23,23,22,22	;b0-bf
;	dt	21,21,20,19,19,18,18,17,17,16,15,15,14,14,13,12	;c0-cf
;	dt	12,11,10,10,0f9,0f8,0f8,0f7,0f6,0f6,0f5,0f4,0f3,0f2,0f2,0f1	;d0-df
;	dt	0f0,0a1,0a2,0a3,0a3,0a4,0a5,0a6,0a7,0a8,0a9,0de,0de,0de,0de,0de	;e0-ef
;240
;	dt	0de,0de,0de,0de,0de,0de,0de,0de,0de,0de,0de,0de,0de,0de,0de	;f0-fe "Lo"
;	dt	0eeh	;5 "oo"

;pevodn tabulka pro termistor 10k B=3900K, pull-up 3k9
;	dt	0aah	; "--"
;	dt	0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc	;01-0f "Hi"
;	dt	0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc	;10-1f "Hi"
;	dt	0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,99,98,97,96,95,94,93,92	;20-2f "Hi"
;	dt	92,91,90,89,88,88,87,86,85,85,84,83,82,82,81,80	;30-3f
;	dt	80,79,79,78,77,77,76,75,75,74,74,73,73,72,71,71	;40-4f
;80
;	dt	70,70,69,69,68,68,67,67,66,66,65,65,64,64,63,63	;50-5f
;	dt	62,62,61,61,61,60,60,59,59,58,58,57,57,56,56,56	;60-6f
;	dt	55,55,54,54,53,53,53,52,52,51,51,50,50,50,49,49	;70-7f
;	dt	48,48,47,47,47,46,46,46,45,45,44,44,44,43,43,42	;80-8f
;	dt	42,42,41,41,40,40,40,39,39,38,38,38,37,37,36,36	;90-9f
;160
;	dt	36,35,35,34,34,33,33,33,32,32,31,31,31,30,30,29	;a0-af
;	dt	29,28,28,28,27,27,26,26,25,25,24,24,24,23,23,22	;b0-bf
;	dt	22,21,21,20,20,19,19,18,18,17,17,16,16,15,15,14	;c0-cf
;	dt	14,13,12,12,11,11,10,10,0f9,0f8,0f8,0f7,0f6,0f6,0f5,0f4	;d0-df
;	dt	0f4,0f3,0f2,0f1,0f0,0f0,0a1,0a2,0a3,0a4,0a5,0a6,0a7,0a8,0a9,0de	;e0-ef
;240
;	dt	0de,0de,0de,0de,0de,0de,0de,0de,0de,0de,0de,0de,0de,0de,0de	;f0-fe "Lo"
;	dt	0eeh	;5 "oo"

;termistor 10k (K164NK010) B=4300K
	dt	0aah	; "--"
	dt	0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc	;01-0f "Hi"
	dt	0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc,0bc	;10-1f "Hi"
	dt	99,98,97,96,95,94,94,93,92,91,90,89,88,87,87,86	;20-2f "Hi"
	dt	85,84,84,83,82,82,81,80,80,79,78,78,77,76,76,75	;30-3f
	dt	75,74,74,73,73,72,71,71,70,70,69,69,68,68,67,67	;40-4f
;80
	dt	66,66,65,65,64,64,64,63,63,62,60,61,61,60,60,60	;50-5f
	dt	59,59,58,58,57,57,57,56,56,55,55,55,54,54,53,53	;60-6f
	dt	53,52,52,51,51,51,50,50,50,49,49,48,48,48,47,47	;70-7f
	dt	47,46,46,45,45,45,44,44,44,43,43,42,42,42,41,41	;80-8f
	dt	41,40,40,39,39,39,38,38,38,37,37,37,36,36,35,35	;90-9f
;160
	dt	35,34,34,34,33,33,32,32,32,31,31,31,30,30,29,29	;a0-af
	dt	29,28,28,27,27,27,26,26,25,25,24,24,24,23,23,22	;b0-bf
	dt	22,21,21,21,20,20,19,19,18,18,17,17,16,16,15,15	;c0-cf
	dt	14,14,13,13,12,12,11,11,10,0f9,0f9,0f8,0f8,0f7,0f6,0f6	;d0-df
	dt	0f5,0f4,0f4,0f3,0f2,0f1,0f1,0f0,0a1,0a2,0a3,0a4,0a5,0a6,0a7,0a8	;e0-ef
;240
	dt	0a9,0de,0de,0de,0de,0de,0de,0de,0de,0de,0de,0de,0de,0de,0de	;f0-fe "Lo"
	dt	0eeh	;5 "oo"


		org 3f0h
	retlw	.100	;dlka nabjen x 6 minut

		org 3f1h
	retlw	085h	;nabjen se vypne pi 45C

		org 3ffh
	retlw	054h	;OSCCAL (pro kad kus IO jin!!!

; initialize eeprom locations

		ORG	0x2100
	DE	"v10b VH2009 Nabijecka (meri cas a teplotu). Vypne GP1 pri prekroceni casu nebo teploty. "
	DE	"Jenom teplomer kdyz GP5=L.  10h 45 deg C"



		END                       ; directive 'end of program'

